package component

import androidx.compose.animation.AnimatedContent
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Clear
import androidx.compose.material.icons.filled.Menu
import androidx.compose.material3.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.semantics.isTraversalGroup
import androidx.compose.ui.semantics.semantics
import androidx.compose.ui.semantics.traversalIndex
import androidx.compose.ui.unit.dp
import data.ENumber

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun Search(eNumbers: List<ENumber>) {
    var text by rememberSaveable { mutableStateOf("") }
    var expanded by rememberSaveable { mutableStateOf(false) }

    Column (
        Modifier
            .fillMaxWidth()
            .padding(if (!expanded) 8.dp else 0.dp)
            .semantics { isTraversalGroup = true }
    ) {
        SearchBar(
            modifier = Modifier
                .align(Alignment.CenterHorizontally)
                .semantics { traversalIndex = 0f },
            query = text,
            onQueryChange = { text = it },
            onSearch = { expanded = false },
            onActiveChange = { expanded = it },
            active = expanded,
            placeholder = { Text("Search E Numbers") },
            leadingIcon = {
                IconButton(onClick = {
                    if (expanded) {
                        expanded = false
                    } else {
                        /* TODO add navigation menu functionality */
                    }
                }) {
                    AnimatedContent(
                        targetState = expanded,
                    ) { targetExpanded ->
                        if (targetExpanded) {
                            Icon(Icons.AutoMirrored.Filled.ArrowBack, "Exit Search")
                        } else {
                            Icon(Icons.Filled.Menu, "Navigation Menu")
                        }
                    }
                }
            },
            trailingIcon = {
                if (text.isNotBlank()) {
                    IconButton(onClick = { text = "" }) {
                        Icon(imageVector = Icons.Filled.Clear, contentDescription = "Clear Search")
                    }
                }
            }
        ) {
            Column(Modifier.verticalScroll(rememberScrollState())) {
                searchSuggestion(query = text, eNumbers = eNumbers).forEach {
                    ListItem(
                        headlineContent = { Text(text = it.number) },
                        modifier = Modifier
                            .clickable {
                                text = it.number
                                expanded = false
                            }
                            .fillMaxWidth()
                            .padding(horizontal = 16.dp, vertical = 4.dp)
                    )
                }
            }
        }
    }
}

private fun searchSuggestion(query: String, eNumbers: List<ENumber>): List<ENumber> {
    if (query.isEmpty()) {
        return eNumbers
    }

    val filtered = eNumbers.filter { eNumber ->
        eNumber.number.contains(query, ignoreCase = true)
    }

    return filtered
}
